import { Component, ElementRef, EventEmitter, HostListener, Input, NgModule, Output, ViewChild } from '@angular/core';
import { CommonModule } from '@angular/common';

const ngxEditorConfig = {
    editable: true,
    spellcheck: true,
    height: 'auto',
    minHeight: '0',
    width: 'auto',
    minWidth: '0',
    translate: 'yes',
    placeholder: 'Enter text here...',
    toolbar: [
        ['bold', 'italic', 'underline', 'strikeThrough', 'superscript', 'subscript'],
        ['justifyLeft', 'justifyCenter', 'justifyRight', 'justifyFull', 'indent', 'outdent'],
        ['cut', 'copy', 'delete', 'removeFormat', 'undo', 'redo'],
        ['paragraph', 'blockquote', 'removeBlockquote', 'horizontalLine', 'orderedList', 'unorderedList'],
        ['link', 'unlink', 'image']
    ]
};

/**
 * @param {?} value
 * @param {?} toolbar
 * @return {?}
 */
function canEnableToolbarOptions(value, toolbar) {
    if (value) {
        if (toolbar['length'] === 0) {
            return true;
        }
        else {
            return toolbar.find(array => {
                return array.includes(value);
            });
        }
    }
    else {
        return false;
    }
}

class NgxEditorComponent {
    /**
     * @param {?} _element
     */
    constructor(_element) {
        this._element = _element;
        this.htmlChange = new EventEmitter();
        this.enableToolbar = false;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set resizer(value) {
        console.log(value);
        if (value === 'basic') {
            this._resizer = value;
        }
        else {
            this._resizer = 'stack';
        }
    }
    /**
     * @return {?}
     */
    get resizer() {
        return this._resizer || 'stack';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set config(value) {
        for (const /** @type {?} */ i in ngxEditorConfig) {
            if (i) {
                if (this[i]) {
                    value[i] = this[i];
                }
                if (!value.hasOwnProperty(i)) {
                    value[i] = ngxEditorConfig[i];
                }
            }
        }
        this._config = value;
    }
    /**
     * @return {?}
     */
    get config() {
        return this._config || ngxEditorConfig;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set html(value) {
        this._html = value;
    }
    /**
     * @return {?}
     */
    get html() {
        return this._html;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    htmlContentChange(value) {
        if (value === '<br>') {
            this.htmlChange.emit('');
        }
        else {
            this.htmlChange.emit(value);
        }
    }
    /**
     * @param {?} commandName
     * @return {?}
     */
    executeCommand(commandName) {
        document.execCommand(commandName, false, null);
    }
    /**
     * @return {?}
     */
    blockQuote() {
        document.execCommand('formatBlock', false, '<blockquote>');
    }
    /**
     * @return {?}
     */
    removeQuote() {
        document.execCommand('formatBlock', false, 'div');
    }
    /**
     * @return {?}
     */
    createLink() {
        const /** @type {?} */ selection = document.getSelection();
        if (selection.anchorNode.parentElement.tagName === 'A') {
            const /** @type {?} */ linkURL = prompt('Enter URL', selection.anchorNode.parentElement.getAttribute('href'));
            if (linkURL) {
                document.execCommand('createLink', false, linkURL);
            }
        }
        else {
            if (selection['type'] === 'None') {
                this.createMessage('No selection made');
            }
            else {
                const /** @type {?} */ linkURL = prompt('Enter URL', 'http://');
                if (linkURL) {
                    document.execCommand('createLink', false, linkURL);
                }
            }
        }
    }
    /**
     * @return {?}
     */
    insertImage() {
        const /** @type {?} */ imageURI = prompt('Enter Image URL', 'http://');
        if (imageURI) {
            const /** @type {?} */ inserted = document.execCommand('insertImage', false, imageURI);
            if (!inserted) {
                this.createMessage('Invalid URL');
            }
        }
    }
    /**
     * @param {?} message
     * @return {?}
     */
    createMessage(message) {
        this.ngxMessage = message;
        setTimeout(() => {
            this.clearMessage();
        }, 5000);
    }
    /**
     * @return {?}
     */
    clearMessage() {
        this.ngxMessage = undefined;
    }
    /**
     * @return {?}
     */
    onFocus() {
        this.enableToolbar = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDocumentClick(event) {
        if (this._element.nativeElement.contains(event.target)) {
            this.enableToolbar = true;
        }
        else {
            this.enableToolbar = false;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    canEnableToolbarOptions(value) {
        return canEnableToolbarOptions(value, this.config['toolbar']);
    }
    /**
     * @param {?} offsetY
     * @return {?}
     */
    resizeTextArea(offsetY) {
        let /** @type {?} */ newHeight = parseInt(this.height, 10);
        newHeight += offsetY;
        this.height = newHeight + 'px';
        this.textArea.nativeElement.style.height = this.height;
    }
    /**
     * @return {?}
     */
    canResize() {
        if (this.resizer === 'basic') {
            return 'vertical';
        }
        return false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.textArea.nativeElement.innerHTML = this.html || '';
        this.height = this.height || this.textArea.nativeElement.offsetHeight;
        document.execCommand('enableObjectResizing', true, true);
    }
}
NgxEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-ngx-editor',
                template: `
    <div class="ngx-editor" id="ngxEditor" [style.width]="config['width']" [style.minWidth]="config['minWidth']">

      <!-- toolbar -->
      <div class="ngx-toolbar">
        <div class="ngx-toolbar-set">
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('bold')" (click)="executeCommand('bold')" title="Bold" [disabled]="!enableToolbar">
            <i class="fa fa-bold" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('italic')" (click)="executeCommand('italic')" title="Italic"
            [disabled]="!enableToolbar">
            <i class="fa fa-italic" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('underline')" (click)="executeCommand('underline')" title="Underline"
            [disabled]="!enableToolbar">
            <i class="fa fa-underline" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('strikeThrough')" (click)="executeCommand('strikeThrough')"
            title="Strikethrough" [disabled]="!enableToolbar">
            <i class="fa fa-strikethrough" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('superscript')" (click)="executeCommand('superscript')"
            title="Superscript" [disabled]="!enableToolbar">
            <i class="fa fa-superscript" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('subscript')" (click)="executeCommand('subscript')" title="Subscript"
            [disabled]="!enableToolbar">
            <i class="fa fa-subscript" aria-hidden="true"></i>
          </button>
        </div>
        <div class="ngx-toolbar-set">
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('justifyLeft')" (click)="executeCommand('justifyLeft')"
            title="Justify Left" [disabled]="!enableToolbar">
            <i class="fa fa-align-left" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('justifyCenter')" (click)="executeCommand('justifyCenter')"
            title="Justify Center" [disabled]="!enableToolbar">
            <i class="fa fa-align-center" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('justifyRight')" (click)="executeCommand('justifyRight')"
            title="Justify Right" [disabled]="!enableToolbar">
            <i class="fa fa-align-right" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('justifyFull')" (click)="executeCommand('justifyFull')"
            title="Justify" [disabled]="!enableToolbar">
            <i class="fa fa-align-justify" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('indent')" (click)="executeCommand('indent')" title="Indent"
            [disabled]="!enableToolbar">
            <i class="fa fa-indent" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('outdent')" (click)="executeCommand('outdent')" title="Outdent"
            [disabled]="!enableToolbar">
            <i class="fa fa-outdent" aria-hidden="true"></i>
          </button>
        </div>
        <div class="ngx-toolbar-set">
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('cut')" (click)="executeCommand('cut')" title="Cut" [disabled]="!enableToolbar">
            <i class="fa fa-scissors" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('copy')" (click)="executeCommand('copy')" title="Copy" [disabled]="!enableToolbar">
            <i class="fa fa-files-o" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('delete')" (click)="executeCommand('delete')" title="Delete"
            [disabled]="!enableToolbar">
            <i class="fa fa-trash" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('removeFormat')" (click)="executeCommand('removeFormat')"
            title="Clear Formatting" [disabled]="!enableToolbar">
            <i class="fa fa-eraser" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('undo')" (click)="executeCommand('undo')" title="Undo" [disabled]="!enableToolbar">
            <i class="fa fa-undo" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('redo')" (click)="executeCommand('redo')" title="Redo" [disabled]="!enableToolbar">
            <i class="fa fa-repeat" aria-hidden="true"></i>
          </button>
        </div>
        <div class="ngx-toolbar-set">
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('paragraph')" (click)="executeCommand('insertParagraph')"
            title="Paragraph" [disabled]="!enableToolbar">
            <i class="fa fa-paragraph" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('blockquote')" (click)="blockQuote()" title="Blockquote"
            [disabled]="!enableToolbar">
            <i class="fa fa-quote-left" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('removeBlockquote')" (click)="removeQuote()" title="Remove Blockquote"
            [disabled]="!enableToolbar">
            <i class="fa fa-quote-right" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('horizontalLine')" (click)="executeCommand('insertHorizontalRule')"
            title="Horizontal Line" [disabled]="!enableToolbar">
            <i class="fa fa-minus" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('orderedList')" (click)="executeCommand('insertUnorderedList')"
            title="Unodered List" [disabled]="!enableToolbar">
            <i class="fa fa-list-ul" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('unorderedList')" (click)="executeCommand('insertOrderedList')"
            title="Ordered List" [disabled]="!enableToolbar">
            <i class="fa fa-list-ol" aria-hidden="true"></i>
          </button>
        </div>
        <div class="ngx-toolbar-set">
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('link')" (click)="createLink()" title="Insert/Edit Link"
            [disabled]="!enableToolbar">
            <i class="fa fa-link" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('unlink')" (click)="executeCommand('unlink')" title="Unlink"
            [disabled]="!enableToolbar">
            <i class="fa fa-chain-broken" aria-hidden="true"></i>
          </button>
          <button class="ngx-editor-button" *ngIf="canEnableToolbarOptions('image')" (click)="insertImage()" title="Insert Image" [disabled]="!enableToolbar">
            <i class="fa fa-picture-o" aria-hidden="true"></i>
          </button>
        </div>
      </div>

      <!-- text area -->
      <div class="ngx-editor-textarea" [attr.contenteditable]="config['editable']" [attr.placeholder]="config['placeholder']" (input)="html = $event.target.innerHTML; htmlContentChange($event.target.innerHTML)"
        [attr.translate]="config['translate']" [attr.spellcheck]="config['spellcheck']" [style.height]="config['height']" [style.minHeight]="minHeight || config['minHeight']"
        [style.resize]="canResize()" (focus)="onFocus()" #ngxTextArea></div>

      <!-- message box -->
      <div class="ngx-editor-message" *ngIf="ngxMessage" (dblclick)="clearMessage()">
        {{ngxMessage}}
      </div>

      <app-ngx-grippie *ngIf="resizer === 'stack'"></app-ngx-grippie>
    </div>
  `,
                styles: [`
    /*
     * editor styles
     */
    .ngx-editor {
      position: relative; }
      .ngx-editor ::ng-deep [contenteditable=true]:empty:before {
        content: attr(placeholder);
        display: block; }
      .ngx-editor .ngx-toolbar {
        background-color: #f5f5f5;
        border-bottom: 1px solid #ddd;
        font-size: 0.8rem;
        padding: 0.2rem;
        border: 1px solid #ddd; }
        .ngx-editor .ngx-toolbar .ngx-toolbar-set {
          display: inline-block;
          border: 1px solid #ddd;
          border-radius: 5px;
          background-color: white; }
          .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button {
            background-color: #f5f5f5;
            background-color: transparent;
            border: 0;
            padding: 0.4rem;
            min-width: 2.5rem;
            float: left;
            border-right: 1px solid #ddd; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:hover {
              cursor: pointer;
              background-color: #f1f1f1;
              -webkit-transition: 0.2s ease;
              transition: 0.2s ease; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:focus, .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button.focus {
              outline: 0; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:last-child {
              border-right: transparent; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:first-child {
              border-top-left-radius: 5px;
              border-bottom-left-radius: 5px; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:last-child {
              border-top-right-radius: 5px;
              border-bottom-right-radius: 5px; }
            .ngx-editor .ngx-toolbar .ngx-toolbar-set .ngx-editor-button:disabled {
              background-color: #f5f5f5;
              pointer-events: none;
              cursor: not-allowed; }
      .ngx-editor .ngx-editor-textarea {
        min-height: 5rem;
        padding: 0.5rem 0.8rem 1rem 0.8rem;
        border: 1px solid #ddd;
        border-top: transparent !important;
        background-color: #fff;
        overflow: auto; }
        .ngx-editor .ngx-editor-textarea:focus, .ngx-editor .ngx-editor-textarea.focus {
          outline: 0; }
        .ngx-editor .ngx-editor-textarea ::ng-deep blockquote {
          margin-left: 1rem;
          border-left: 0.2em solid #dfe2e5;
          padding-left: 0.5rem; }
      .ngx-editor .ngx-editor-message {
        font-size: 80%;
        background-color: #f1f1f1;
        border: 1px solid #ddd;
        border-top: transparent;
        padding: 0 0.5rem;
        padding-bottom: 0.1rem;
        -webkit-transition: 0.5s ease-in;
        transition: 0.5s ease-in; }
  `]
            },] },
];
/**
 * @nocollapse
 */
NgxEditorComponent.ctorParameters = () => [
    { type: ElementRef, },
];
NgxEditorComponent.propDecorators = {
    'htmlChange': [{ type: Output },],
    'textArea': [{ type: ViewChild, args: ['ngxTextArea',] },],
    'editable': [{ type: Input },],
    'spellcheck': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'translate': [{ type: Input },],
    'height': [{ type: Input },],
    'minHeight': [{ type: Input },],
    'width': [{ type: Input },],
    'minWidth': [{ type: Input },],
    'toolbar': [{ type: Input },],
    'resizer': [{ type: Input },],
    'config': [{ type: Input },],
    'html': [{ type: Input },],
    'onDocumentClick': [{ type: HostListener, args: ['document:click', ['$event'],] },],
};

class NgxGrippieComponent {
    /**
     * @param {?} _editorComponent
     */
    constructor(_editorComponent) {
        this._editorComponent = _editorComponent;
        this.oldY = 0;
        this.grabber = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseMove(event) {
        if (!this.grabber) {
            return;
        }
        this._editorComponent.resizeTextArea(event.clientY - this.oldY);
        this.oldY = event.clientY;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseUp(event) {
        this.grabber = false;
    }
    /**
     * @param {?} event
     * @param {?=} resizer
     * @return {?}
     */
    onResize(event, resizer) {
        this.grabber = true;
        this.oldY = event.clientY;
        event.preventDefault();
    }
}
NgxGrippieComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-ngx-grippie',
                template: `
    <div class="ngx-editor-grippie">
      <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" style="isolation:isolate" viewBox="651.6 235 26 5"
        width="26" height="5">
        <g id="sprites">
          <path d=" M 651.6 235 L 653.6 235 L 653.6 237 L 651.6 237 M 654.6 238 L 656.6 238 L 656.6 240 L 654.6 240 M 660.6 238 L 662.6 238 L 662.6 240 L 660.6 240 M 666.6 238 L 668.6 238 L 668.6 240 L 666.6 240 M 672.6 238 L 674.6 238 L 674.6 240 L 672.6 240 M 657.6 235 L 659.6 235 L 659.6 237 L 657.6 237 M 663.6 235 L 665.6 235 L 665.6 237 L 663.6 237 M 669.6 235 L 671.6 235 L 671.6 237 L 669.6 237 M 675.6 235 L 677.6 235 L 677.6 237 L 675.6 237"
            fill="rgb(147,153,159)" />
        </g>
      </svg>
    </div>
  `,
                styles: [`
    .ngx-editor-grippie {
      height: 9px;
      background-color: #f1f1f1;
      position: relative;
      text-align: center;
      cursor: s-resize;
      border: 1px solid #ddd;
      border-top: transparent; }
      .ngx-editor-grippie svg {
        position: absolute;
        top: 1.5px; }
  `]
            },] },
];
/**
 * @nocollapse
 */
NgxGrippieComponent.ctorParameters = () => [
    { type: NgxEditorComponent, },
];
NgxGrippieComponent.propDecorators = {
    'onMouseMove': [{ type: HostListener, args: ['document:mousemove', ['$event'],] },],
    'onMouseUp': [{ type: HostListener, args: ['document:mouseup', ['$event'],] },],
    'onResize': [{ type: HostListener, args: ['mousedown', ['$event'],] },],
};

class NgxEditorModule {
}
NgxEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [NgxEditorComponent, NgxGrippieComponent],
                exports: [NgxEditorComponent, NgxGrippieComponent]
            },] },
];
/**
 * @nocollapse
 */
NgxEditorModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { NgxEditorModule, NgxEditorComponent as ɵa, NgxGrippieComponent as ɵb };
//# sourceMappingURL=ngx-editor.js.map
